package com.ether;

import gov.noaa.pmel.sgt.JPane;
import gov.noaa.pmel.util.Dimension2D;
import gov.noaa.pmel.util.Point2D;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import com.medias.megapoli.utils.MegapoliInitialisation;

import javax.swing.*;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.util.Calendar;
import java.util.Locale;
import java.util.ResourceBundle;

/**
 * @author vmipsl
 * @date 05 july 2011
 */
public class EtherPlotServiceImpl
        implements EtherPlotService
{
    public static Integer getMaxWidth()
    {
        final Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        return Math.min( Double.valueOf( screenSize.getWidth() ).intValue(), MAX_WIDTH );
    }

    public static Integer getMaxHeight()
    {
        final Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        return Math.min( Double.valueOf( screenSize.getHeight() ).intValue(), MAX_HEIGHT );
    }

    /**
     * Create the main JPane with the 3 jPanes
     *
     * @param megapoliPlot
     * @param locale
     * @return
     */
    @NotNull
    public BufferedImage createMainPane( @NotNull final MegapoliPlot megapoliPlot, @Nullable final Locale locale )
    {
        final Integer maxWidth = getMaxWidth();
        final Integer maxHeight = getMaxHeight();

        final JPane jPane = new JPane( "Main jPane", new Dimension( maxWidth, maxHeight ) );
        jPane.setLayout( new BorderLayout() );
        jPane.setBackground( Color.white );
        jPane.setOpaque( true );

        // Top Pane
        final JPane jPaneTop = createTopPane( megapoliPlot );
        jPane.add( jPaneTop, BorderLayout.NORTH );

        final Integer valuesNumber = megapoliPlot.getData().getYArray().length;

        // Bottom Pane
        final JPane jPaneBottom = createBottomPane( valuesNumber, locale );
        jPane.add( jPaneBottom, BorderLayout.SOUTH );

        // Center Pane
        final Integer centerHeight = maxHeight - jPaneTop.getHeight() - jPaneBottom.getHeight();
        final JPane jPaneCenter;
        if( valuesNumber > 0 )
            jPaneCenter = createCenterPane( megapoliPlot, maxWidth, centerHeight );
        else
            jPaneCenter = createEmptyCenterPane( locale );
        jPane.add( jPaneCenter, BorderLayout.CENTER );


        final JPanelToImageManager jPanelToImageManager = new JPanelToImageManager( jPane );
        return jPanelToImageManager.saveAsImage();
    }

    /**
     * Create the top JPane with the logos and title
     *
     * @param megapoliPlot
     * @return
     */
    @NotNull
    public JPane createTopPane( @Nullable final MegapoliPlot megapoliPlot )
    {
        // Logos
        final ImageIcon logoMegapoli = new ImageIcon( megapoliPlot.getLogoMegapoli() );
        final JLabel jLabelLogoMegapoli = new JLabel( logoMegapoli );

        final ImageIcon logoEther = new ImageIcon( megapoliPlot.getLogoEther() );
        final JLabel jLabelLogoEther = new JLabel( logoEther );

        // Max height
        final Integer maxHeights = Math.max( logoMegapoli.getIconHeight(), logoEther.getIconHeight() );

        // jPane
        final JPane jPaneTop = new JPane( "Top jPane", new Dimension( 0, maxHeights ) );
        jPaneTop.setLayout( new BorderLayout() );

        jPaneTop.add( jLabelLogoMegapoli, BorderLayout.LINE_START );
        jPaneTop.add( jLabelLogoEther, BorderLayout.LINE_END );


        // Title
        final String formatedTitle = formatTitle( megapoliPlot.getTitle() );
        final JLabel jLabelTitle = new JLabel( formatedTitle, JLabel.CENTER );
        final Font police = new Font( "Arial", Font.BOLD, TITLE_FONT_SIZE );
        jLabelTitle.setFont( police );
        jPaneTop.add( jLabelTitle, BorderLayout.CENTER );

        return jPaneTop;
    }

    /**
     * Create the center JPane with the graph
     *
     * @param megapoliPlot
     * @param maxWidth
     * @param maxHeight
     * @return
     */
    @NotNull
    public JPane createCenterPane( @NotNull final MegapoliPlot megapoliPlot, @NotNull final Integer maxWidth, @NotNull final Integer maxHeight )
    {
        final Integer plotWidth = Math.min( maxWidth - MARGIN_LEFT_RIGHT, maxWidth );
        final Integer plotHeight = Math.min( maxHeight - MARGIN_LEFT_RIGHT, maxHeight );

        // Graph
        final JPlotLayoutEther jPlotLayout = new JPlotLayoutEther( false, megapoliPlot.isXTime(), false, "Trajectory data", null, megapoliPlot.isLegendToHide() );
        jPlotLayout.setTitles( "", "", "" );

        // Replace legend
        final String legend = megapoliPlot.getData().getTitle();
        final Dimension2D layerSizeP = jPlotLayout.getLayerSizeP();
        final double factorForPlotWidthInPhysicalUnits = layerSizeP.getWidth() / Double.valueOf( plotWidth );
        final double legendSizeInPhysicalUnits = legend.length() * factorForPlotWidthInPhysicalUnits * 7;
        jPlotLayout.setKeyLocationP( new Point2D.Double( layerSizeP.getWidth() - legendSizeInPhysicalUnits, layerSizeP.getHeight() - 0.1 ) );

        // Add data and legend
        jPlotLayout.addData( megapoliPlot.getData(), legend );
        jPlotLayout.setSize( plotWidth, plotHeight );
        jPlotLayout.setVisible( true );

        jPlotLayout.addNotify();
        jPlotLayout.validate();
        jPlotLayout.setOpaque( true );

        // TODO : remove this change to bufferedImage to insert directly the jPlotLayout !!!!
        final BufferedImage bufferedImage = new BufferedImage( jPlotLayout.getWidth(), jPlotLayout.getHeight(), BufferedImage.TYPE_INT_RGB );
        final Graphics2D graphics2D = bufferedImage.createGraphics();
        jPlotLayout.draw( graphics2D );

        final ImageIcon imageIcon = new ImageIcon( bufferedImage );
        final JLabel jLabelPlot = new JLabel( imageIcon, JLabel.CENTER );
        jLabelPlot.setBorder( BorderFactory.createLineBorder( Color.black, 1 ) );

        final JPane jPaneCenter = new JPane( "Center jPane", new Dimension( jPlotLayout.getWidth(), jPlotLayout.getHeight() ) );
        jPaneCenter.setLayout( new GridBagLayout() );
        jPaneCenter.add( jLabelPlot,
                new GridBagConstraints( 0, 0, 1, 1, 0, 0,
                        GridBagConstraints.CENTER,
                        GridBagConstraints.CENTER,
                        new Insets( 0, 0, 0, 0 ), 0, 0 ) );

        return jPaneCenter;
    }

    /**
     * Create the center JPane with no graph (if no value)
     *
     * @param locale
     * @return
     */
    public JPane createEmptyCenterPane( @Nullable final Locale locale )
    {
        final ResourceBundle bundle = WebHelper.getBundle( locale );
        final String messageData = bundle.getString( "plot.noData" );

        final JPane jPaneCenter = new JPane();
        jPaneCenter.setLayout( new BorderLayout() );

        final JLabel jLabelData = new JLabel( messageData, JLabel.CENTER );
        final Font police = new Font( "Arial", Font.BOLD, FONT_SIZE );
        jLabelData.setFont( police );

        jPaneCenter.add( jLabelData, BorderLayout.CENTER );
        return jPaneCenter;
    }

    /**
     * Create the bottom JPane with the text "published .. (date)"
     *
     * @param dataNumber : the number of datas extracted from the base
     * @param locale
     * @return
     */
    @NotNull
    public JPane createBottomPane( @Nullable final Integer dataNumber, @Nullable final Locale locale )
    {
        final ResourceBundle bundle = WebHelper.getBundle( locale );

        final JPane jPaneBottom = new JPane();
        jPaneBottom.setLayout( new BorderLayout() );

        // Published
        final Calendar calendar = Calendar.getInstance();
        final String messagePublished = bundle.getString( "plot.published" );
        final JLabel jLabel = new JLabel( messagePublished + ' ' + DateHelper.formatDate( calendar.getTime(), DateHelper.ENGLISH_DATE_PATTERN ) + " " );
        final Font police = new Font( "Arial", Font.BOLD, FONT_SIZE );
        jLabel.setFont( police );
        jPaneBottom.add( jLabel, BorderLayout.EAST );

        if( null != dataNumber )
        {
            final String messageDataNumber = bundle.getString( "plot.dataNumber" );
            final JLabel jLabelDataNumber = new JLabel( messageDataNumber + ' ' + dataNumber );
            jLabelDataNumber.setFont( police );
            jPaneBottom.add( jLabelDataNumber, BorderLayout.WEST );
        }

        return jPaneBottom;
    }

    /**
     * Create an error JPane when plot can be created
     *
     * @param mainError
     * @param errorText
     * @param locale
     * @return
     */
    @NotNull
    public BufferedImage createErrorPane( @NotNull final String mainError, @Nullable final String errorText, @Nullable final Locale locale )
    {
        // Image
        final ImageIcon errorImage = new ImageIcon( MegapoliInitialisation.pathImages + "/erreur.gif" );
        final JLabel jLabelErrorImage = new JLabel( errorImage );

        // Text
        final ResourceBundle bundle = WebHelper.getBundle( locale );
        final String mainMessage = bundle.getString( "plot.errorMessage" );

        final JLabel jLabelError;
        if( null != errorText )
            jLabelError = new JLabel( "<html>" + mainMessage + "<br><br>" + mainError + "<br>" + errorText + "</html>", JLabel.CENTER );
        else
            jLabelError = new JLabel( "<html>" + mainMessage + "<br><br>" + mainError + "</html>", JLabel.CENTER );

        final Font police = new Font( "Arial", Font.BOLD, ERROR_FONT_SIZE );
        jLabelError.setFont( police );

        // jPane
        final JPane jPane = new JPane( "jPane", new Dimension( MAX_WIDTH, errorImage.getIconHeight() ) );
        jPane.setLayout( new BorderLayout() );
        jPane.setBackground( Color.white );
        jPane.setOpaque( true );
        jPane.add( jLabelErrorImage, BorderLayout.WEST );
        jPane.add( jLabelError, BorderLayout.CENTER );

        final JPanelToImageManager jPanelToImageManager = new JPanelToImageManager( jPane );
        return jPanelToImageManager.saveAsImage();
    }

    @NotNull
    private String formatTitle( @Nullable final String title )
    {
        if( null == title )
            return "";
        else
        {
            final String formatTitle = title.replaceAll( "\\n", "<br>" );
            return "<html>" + formatTitle + "</html>";
        }
    }

    private static final Log LOGGER = LogFactory.getLog( EtherPlotServiceImpl.class );

    // Dimensions of the jPanes
    private static final int MAX_WIDTH = 800;
    private static final int MAX_HEIGHT = 700;
    private static final int MARGIN_LEFT_RIGHT = 50;

    private static final int TITLE_FONT_SIZE = 20;
    private static final int FONT_SIZE = 12;
    private static final int ERROR_FONT_SIZE = 10;
}
